//************************************************************************************
//* This sample explains how one can extend HiewLua Lua interpreter with C/C++ code. *
//* It adds to HiewLua support for the following bitwise operations: And, Or, Xor.   *
//* Compile this sample and place the BitOps.dll into the %HIEW%/LuaPlugins folder.  *
//* Next, you can use this DLL in your Lua script, for example:                      *
//*                                                                                  *
//* require("BitOps")                                                                *
//* print(Xor(0xaa, 0x55), And(0xaa, 0x55), Or(0xaa, 0x55))                          *
//************************************************************************************

#include <Windows.h>

//------------------------------------------------------------------------
// Don't we want to be a (kilo)bit smaller?
//
#pragma comment(linker, "/OPT:NOWIN98")


//------------------------------------------------------------------------
// Inlcude Lua headers
// Please ensure that HiewLua SDK path is added to project settings as an additional
// directory for compliler and linker. Link with HiewLua.lib.
//
#include <Lua.hpp>

//------------------------------------------------------------------------
// DLL entry point
// Note: since we don't need anything special here, we can simply omit the whole
// function
//
#if 0

BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    return TRUE;
}

#endif

//------------------------------------------------------------------------
// The functions we add to Lua interpreter
// Note: HiewLua uses CALL_CONV (which resolves to __stdcall) calling convention
// everywhere so our functions must be of type CALL_CONV, too.
// You can set __stdcall as the default calling convention in project options
// to spare some typing, however, this looks less portable.
//
static int CALL_CONV lua_And(PLUASTATE L)
{
    /* we expect two unsigned integers to be placed on the stack */
    DWORD Num1 = luaL_checkdword(L, 1);
    DWORD Num2 = luaL_checkdword(L, 2);

    /* yeah, we got the numbers so we can perform the operation */
    DWORD Result = Num1 & Num2;

    /* return the result to Lua */
    lua_pushinteger(L, Result);
    return 1;
}

static int CALL_CONV lua_Or(PLUASTATE L)
{
    /* we expect two unsigned integers to be placed on the stack */
    DWORD Num1 = luaL_checkdword(L, 1);
    DWORD Num2 = luaL_checkdword(L, 2);

    /* yeah, we got the numbers so we can perform the operation */
    DWORD Result = Num1 | Num2;

    /* return the result to Lua */
    lua_pushinteger(L, Result);
    return 1;
}

static int CALL_CONV lua_Xor(PLUASTATE L)
{
    /* we expect two unsigned integers to be placed on the stack */
    DWORD Num1 = luaL_checkdword(L, 1);
    DWORD Num2 = luaL_checkdword(L, 2);

    /* yeah, we got the numbers so we can perform the operation */
    DWORD Result = Num1 ^ Num2;

    /* return the result to Lua */
    lua_pushinteger(L, Result);
    return 1;
}


//------------------------------------------------------------------------
// Register our functions.
// Note: this function gets caled when a Lua script issues the require("BitOps") call
//
extern "C" __declspec(dllexport) int CALL_CONV luaopen_BitOps(PLUASTATE L)
{
    lua_register(L, "And", lua_And);
    lua_register(L,  "Or", lua_Or);
    lua_register(L, "Xor", lua_Xor);
    return 0;
}